"""
Test case for the Latex2MathML markdown extension.
"""

import re
from unittest import TestCase

from markdown import Markdown
from l2m4m import LaTeX2MathMLExtension


class TestExtension(TestCase):
    """
    Tests for the Latex2MathML markdown extension.
    """

    def setUp(self):
        self.markdown = Markdown(extensions=[LaTeX2MathMLExtension()])

    # pylint: disable=invalid-name
    def assertHTMLEqualsMarkdown(self, html, markdown):
        """
        Assert that the specified Markdown coverts to HTML identical to the
        specified HTML. Whitespace surrounding newlines in the specified
        HTML are discarded.
        """
        self.assertEqual(
            re.sub(r"\s*\n\r?\s*", "", html),
            self.markdown.convert(markdown)
        )

    def test_no_latex_no_conversion(self):
        """
        Assert that markdown conversion still works as expected.
        """
        self.assertHTMLEqualsMarkdown(
            "<p>Spam, bacon, eggs.</p>",
            "Spam, bacon, eggs."
        )

    def test_inline_latex_converted(self):
        """
        Assert that inline LaTeX math is converted as expected.
        """
        self.assertHTMLEqualsMarkdown(
            """
            <p>
                Spam, bacon, eggs. <math display="inline" xmlns="
                                        http://www.w3.org/1998/Math/MathML">
                    <mrow>
                        <mi>a</mi>
                        <mo>&#x0002B;</mo>
                        <mi>b</mi>
                        <mo>&#x0003D;</mo>
                        <mi>c</mi>
                    </mrow>
                </math>. Sausage, ham.
            </p>
            """,
            "Spam, bacon, eggs. $a + b = c$. Sausage, ham."
        )

    def test_block_latex_converted(self):
        """
        Assert that Latex math blocks are converted as expected.
        """
        self.assertHTMLEqualsMarkdown(
            """
            <math display="block" xmlns="http://www.w3.org/1998/Math/MathML">
                <mrow>
                    <mi>a</mi>
                    <mo>&#x0002B;</mo>
                    <mi>b</mi>
                    <mo>&#x0003D;</mo>
                    <mi>c</mi>
                </mrow>
            </math>
            """,
            "$$a + b = c$$"
        )

    def test_block_multiline_latex_converted(self):
        """
        Assert that LaTeX math blocks of multiple lines are converted as
        expected.
        """
        self.assertHTMLEqualsMarkdown(
            """
            <math display="block" xmlns="http://www.w3.org/1998/Math/MathML">
                <mrow>
                    <mi>a</mi>
                    <mo>&#x0002B;</mo>
                    <mi>b</mi>
                    <mo>&#x0003D;</mo>
                    <mi>c</mi>
                    <mspace linebreak="newline"></mspace>
                    <mi>c</mi>
                    <mo>&#x0003D;</mo>
                    <mi>a</mi>
                    <mo>&#x02212;</mo>
                    <mi>b</mi>
                </mrow>
            </math>
            """,
            """
            $$
            a + b = c\\\\
            c = a - b
            $$
            """
        )



"""
Definition of the Latex2MathML extension.
"""

import re

from latex2mathml import converter

from markdown import Extension
from markdown.blockprocessors import BlockProcessor
from markdown.inlinepatterns import Pattern


# pylint: disable=too-few-public-methods
class LaTeX2MathMLExtension(Extension):
    """
    The Latex2MathMLExtension class.
    """

    _RE_LATEX = r"\$([^$]+)\$"

    # pylint: disable=invalid-name
    def extendMarkdown(self, md):
        """
        Extend the specified markdown instance with a pattern for inline
        LaTex and a parser for blocks of LaTeX, to be converted to MathML.
        """
        md.inlinePatterns.register(
            LatexPattern(self._RE_LATEX), "latex-inline", 100)
        md.parser.blockprocessors.register(
            LatexBlockProcessor(md.parser), 'latex-block', 100)


# pylint: disable=too-few-public-methods
class LatexPattern(Pattern):
    """
    A pattern for inline LaTeX.
    """

    # pylint: disable=invalid-name
    def handleMatch(self, m):
        """
        Convert inline LaTeX to MathML.
        """
        return converter.convert_to_element(m.group(2))


# pylint: disable=too-few-public-methods
class LatexBlockProcessor(BlockProcessor):
    """
    A processor for blocks of LaTeX.
    """

    _RE_LATEX_START = r"^\s*\${2}"
    _RE_LATEX_END = r"\${2}\s*$"

    def test(self, _, block):
        """
        Indicated whether the specified block starts a block of LaTeX.
        """
        return re.search(self._RE_LATEX_START, block)

    def run(self, parent, blocks):
        """
        Convert all subsequent blocks of LaTeX to MathML. Cancel conversion
        in case no ending block is found.
        """
        text = ""

        for i, block in enumerate(blocks):
            text = text + block

            if re.search(self._RE_LATEX_END, block):
                for j in range(0, i + 1):
                    blocks.pop(j)
                break
            if i >= len(blocks) - 1:
                # no ending block is found
                return False

        text = re.sub(self._RE_LATEX_START, "", text)
        text = re.sub(self._RE_LATEX_END, "", text)

        converter.convert_to_element(text, display="block", parent=parent)

        return True
